
#include "los.h"

typedef double DOUBLE;

/********************
 * global variables *
 ********************/

extern RASTER               elev_raster;          /* elevation data location    */
extern RASTER               *prod_raster /*= NULL*/;  /* LOS analyis data location  */
extern PARAM_CALC           param;                /* calculations location      */
extern HANDLES              handle;               /* handles to working buffers */
extern SETTINGS_TYPE settings;     /* user entered settings                    */
extern DIRECTORY home_dir;

/************************
 * file scope variables *
 ************************/

static RASTER        *backgnd_raster=NULL; /* for saving basemap         */
static FILE_SPEC     prod_file_spec;       /* product file spec          */

void do_los() {
   long                loop;               /* loop counter               */
   char  pHMEM         prod1_ptr;          /* ptr to product raster data */
   RASTER** ptr;
   extern RASTER* prod_raster;

   /* free product memory */
   if (prod_raster != (RASTER *)NULL)
   {
     /* status_update ("old product being destroyed"); */
      /* xvt_scr_set_busy_cursor(); */
      product_destruct(NULL, TRUE);
      prod_raster = (RASTER *)NULL;
   }

   /* check if DTED has been loaded */
   if ( elev_raster.data_matrix.ghandle == (GHANDLE)NULL ) {
      printf("DTED was never loaded"); printf("\n");
      return;
   }

   /* construct product raster */
   ptr = &prod_raster;
   if (raster_construct(NULL, ptr/*&prod_raster*/) != STAT_SUCCESS) {
      printf("Out of Memory - Unable to construct product raster"); printf("\n");
      return;
   }

   raster_assign (&prod_raster);

   /* initialize user input parameters */
   init_ui_params (&prod_raster);

   /* allocate memory for visibility mask */
   prod_raster->data_matrix.ghandle = (GHANDLE) xvt_gmem_alloc
                                      ((long)(prod_raster->data_matrix.width *
                                      prod_raster->data_matrix.height *
                                      sizeof(char)));

   if (prod_raster->data_matrix.ghandle == (GHANDLE)NULL)
      printf("Out of memory in do_product()\n");
   prod1_ptr = (char*) prod_raster->data_matrix.ghandle;
   for (loop=0; loop < (prod_raster->data_matrix.width *
                        prod_raster->data_matrix.height); loop++)
      prod1_ptr[loop] = (char)ZERO;

#if 0
   /* open the status dialog for LOS product */
   if (settings.profile)
      xvt_scr_set_busy_cursor();
   else
      invoke (LOS_DLG, (long)FALSE);
#endif

   /* build elevation arrays for each radial */
   make_radials();
}

/************************************************************************
 *   FUNCTION     : product_destruct
 *
 *   PURPOSE      : saves a product and provides the option to remove it
 *                  from memory along with any other allocated memory.
 *
 *   INPUTS       : fptr - file pointer (if not NULL, the product is saved)
 *                  destroy - if TRUE, product memory is released
 *
 *   RETURN VALUE : TRUE on error, FALSE otherwise
 *
 *   CALLED FROM  : do_file_menu in module LOSMENU1.C
 *                  kill_windows in module LOSMENU1.C
 *
 *   CALLS        : available_structure {MUSE LIB}
 *                  raster_destruct {MUSE LIB}
 *                  update_view_menu in module LOSMENUS.C
 *
 ************************************************************************/

#ifdef PROTO
BOOLEAN product_destruct(FILE *fptr, BOOLEAN destroy)
#else
BOOLEAN product_destruct(fptr, destroy)
FILE *fptr;
BOOLEAN destroy;
#endif
{
   AVAILABLE *av=NULL;                  /* info about avail. products     */
   MAP_DOC   *mapdoc;                   /* basemap map document           */
   BOOLEAN   min_scale, max_scale;      /* used to reset view menu        */
   DOUBLE    dbl_num;                   /* used in swapping bytes         */

   long      loop;                      /* loop counter                   */
   long      lon_num;                   /* used in swapping bytes         */
   char      pHMEM prfl_colr;           /* ptr to profile color buffer    */
   short     pHMEM prfl_hgts;           /* ptr to profile height buffer   */
   short     prior_mask;                /* product priority mask          */
   short     prod_type;                 /* product type mask              */
   short     product;                   /* product exists variable        */
   short     prof;                      /* profile exists variable        */
   short     shrt_num;                  /* used in swapping bytes         */
   short     profile;                   /* flag used to check prfl exists */
   char      header[80], ch;            /* header string for each file    */

   extern SETTINGS_TYPE settings;       /* user settings                  */
   extern MAP_TYPE      mapinfo;        /* non-muse map information       */
   extern WIN_INFO      wininfo;        /* non-muse window infomation     */
#if 0
   /* get a pointer to the MUSE structure */
   available_structure (&av);
   if (av->windows->current_map == (WINDOW)NULL)
      xvt_dm_post_fatal_exit("No basemap is open");

   /* get basemap map document pointers */
   mapdoc = (MAP_DOC *)xvt_vobj_get_data(av->windows->current_map);
   if (mapdoc == (MAP_DOC *)NULL)
      xvt_dm_post_fatal_exit("Unable to access the basemap map document");

   /* check to see if we should save */
   if (fptr != (FILE *)NULL)
   {
      /* write out the file header */
      strcpy (header, HEADER);
      fwrite (header, (long)(HEADER_LEN * sizeof(char)), 1, fptr);

      /* write extra chars */
      ch = (char)32;
      fwrite (&ch, sizeof(char), 1, fptr);
      ch = (char)26;
      fwrite (&ch, sizeof(char), 1, fptr);

      /* save product raster */
      if (raster_destruct (fptr, FALSE, &prod_raster) != STAT_SUCCESS)
         return (BOOLEAN)TRUE;

      /* save out user settings */
      dbl_num = settings.start;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.end;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.observer.loc.x;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.observer.loc.y;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.observer.height;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.target.loc.x;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.target.loc.y;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.target.height;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.target.width;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = settings.range;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      lon_num = settings.vert_acc;
      if (big_endian_machine())
         lon_num = swap_long (lon_num);
      if (fwrite (&lon_num, sizeof(long), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      prior_mask = (short)settings.prior_mask;
      prod_type  = (short)settings.prod_type;
      product    = (short)settings.product;
      prof       = (short)settings.profile;
      if (big_endian_machine())
      {
         prior_mask = swap_short (prior_mask);
         prod_type  = swap_short (prod_type);
         product    = swap_short (product);
         prof       = swap_short (prof);
      }

      if (fwrite (&prior_mask, sizeof(short), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;
      if (fwrite (&prod_type, sizeof(short), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;
      if (fwrite (&product, sizeof(short), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;
      if (fwrite (&prof, sizeof(short), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      /* save out the map information structure */
      dbl_num = mapinfo.dist;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = mapinfo.prof_dist;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      /* save out the los or profile parameters */
      dbl_num = param.radial_spacing;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = param.grid_size_m;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = param.grid_size_dd;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = (double)param.x_res_m;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = (double)param.y_res_m;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      dbl_num = (double)param.radial_step_size;
      if (big_endian_machine())
         dbl_num = swap_double (dbl_num);
      if (fwrite (&dbl_num, sizeof(DOUBLE), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      lon_num = param.num_radials;
      if (big_endian_machine())
         lon_num = swap_long (lon_num);
      if (fwrite (&lon_num, sizeof(long), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      lon_num = param.nhts_in_radial;
      if (big_endian_machine())
         lon_num = swap_long (lon_num);
      if (fwrite (&lon_num, sizeof(long), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      lon_num = param.max_nhts_in_radial;
      if (big_endian_machine())
         lon_num = swap_long (lon_num);
      if (fwrite (&lon_num, sizeof(long), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      lon_num = param.radial_length;
      if (big_endian_machine())
         lon_num = swap_long (lon_num);
      if (fwrite (&lon_num, sizeof(long), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      shrt_num = param.radial_count;
      if (big_endian_machine())
         shrt_num = swap_short (shrt_num);
      if (fwrite (&shrt_num, sizeof(short), 1, fptr) != (size_t)1)
         return (BOOLEAN)TRUE;

      /* check if profile exists */
      if ((handle.prfl != (GHANDLE)NULL) && (handle.hgts != (GHANDLE)NULL))
      {
         profile = (short)TRUE;
         if (big_endian_machine())
            profile = swap_short(profile);
         if (fwrite (&profile, sizeof(short), 1, fptr) != (size_t)1)
            return (BOOLEAN)TRUE;

         prfl_colr = (char  pHMEM) xvt_gmem_lock (handle.prfl);
         prfl_hgts = (short pHMEM) xvt_gmem_lock (handle.hgts);
         if ((prfl_colr == (char pHMEM)NULL) || (prfl_hgts == (short pHMEM)NULL))
            xvt_dm_post_fatal_exit("Unable to lock down profile memory buffers in product_destruct");

         /* write profile to file */
         for (loop = 0; loop < param.nhts_in_radial; loop++)
         {
            fwrite (&(prfl_colr[loop]), sizeof(char), 1, fptr);

            shrt_num = (short)prfl_hgts[loop];
            if (big_endian_machine())
               shrt_num = swap_short(shrt_num);
            if (fwrite (&shrt_num, sizeof(short), 1, fptr) != (size_t)1)
               return (BOOLEAN)TRUE;
         }
       
         if (xvt_gmem_unlock(handle.hgts) == FALSE)
            xvt_dm_post_fatal_exit("Unable to unlock profile height buffer");
         if (xvt_gmem_unlock(handle.prfl) == FALSE)
            xvt_dm_post_fatal_exit("Unable to unlock profile color buffer");
      }
      else
      {
         profile = (short)FALSE;
         if (big_endian_machine())
            profile = swap_short(profile);
         fwrite (&profile, sizeof(short), 1, fptr);
      }
   }
#endif
   if (destroy)
   {
      /* remove profile from screen and memory */
#if 0
      if (wininfo.lookupTable503[0] != (WINDOW)NULL)
      {
         xvt_vobj_destroy(wininfo.lookupTable503[0]);
         xvt_app_process_pending_events();

         xvt_R3_set_front_window(wininfo.lookupTable501[0]); 
		/* xvt_scr_set_focus_vobj(wininfo.lookupTable501[0]);  -- set_focus does not raise window */
		/* xvt_vobj_raise(wininfo.lookupTable501[0]);          -- note: raise is illegal on controls */
         xvt_app_process_pending_events();
         
         min_scale = (BOOLEAN)(mapinfo.scale != MIN_ZOOM);
         max_scale = (BOOLEAN)(mapinfo.scale != MAX_ZOOM);
         update_view_menu(T, T, T, F, T, max_scale, min_scale, T, F, T, F, F, mapinfo.boresight);
         xvt_menu_update(TASK_WIN);
         xvt_app_process_pending_events();
         wininfo.lookupTable503[0] = (WINDOW)NULL;
      }

      /* free profile color buffer if it is still allocated */
      if (handle.prfl != (GHANDLE)NULL)
      {
         if (xvt_gmem_free (handle.prfl) == FALSE)
            printf("Unable to release profile color buffer\n");
         handle.prfl = (GHANDLE)NULL;
      }

      /* free profile height buffer if it is still allocated */
      if (handle.hgts != (GHANDLE)NULL)
      {
         if (xvt_gmem_free (handle.hgts) == FALSE)
            printf("Unable to release profile heights buffer\n");
         handle.hgts = (GHANDLE)NULL;
      }
#endif

      /* free product raster */
      if (prod_raster != (RASTER *)NULL)
      {
         if (raster_destruct (NULL, TRUE, &prod_raster) != STAT_SUCCESS)
            printf("Unable to release product memory\n");
         prod_raster = (RASTER *)NULL;
      }
#if 0
      /* destroy backgnd raster */
      if (backgnd_raster != (RASTER *)NULL)
      {
         restore_basemap (&backgnd_raster);
         backgnd_raster = (RASTER *)NULL;
      }

      /* release memory for the cubic spline handles */
      kill_cspline_handles();
#endif

      settings.product   = FALSE;
#if 0
      mapdoc->needs_save = FALSE;
#endif
   }

   return (BOOLEAN)FALSE;
}
#if 0
/************************************************************************
 *   FUNCTION     : product_construct
 *
 *   PURPOSE      : loads a basemap (if necessary) and a product
 *                  to display.
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : BOOLEAN - TRUE if an error occurred, FALSE otherwise
 *
 *   CALLED FROM  : do_file_menu in module LOSMENU1.C
 *
 *   CALLS        : available_structure {MUSE LIB}
 *                  raster_destruct {MUSE LIB}
 *                  raster_construct {MUSE LIB}
 *                  basemap_new_bitmap {MUSE LIB}
 *                  basemap_match_geometry {MUSE LIB}
 *                  raster_xform {MUSE LIB}
 *                  popen_struct {MUSE LIB}
 *                  ptranslate_struct {MUSE LIB}
 *                  pclose_struct {MUSE LIB}
 *                  raster_draw {MUSE LIB}
 *                  status_update in module LOSMISC.C
 *                  fix_settings in module LOSSET.C
 *                  product_destruct in module DO_PROD.C
 *
 ************************************************************************/

#ifdef PROTO
BOOLEAN product_construct(void)
#else
BOOLEAN product_construct()
#endif
{
   AVAILABLE        *av = NULL;     /* ptr to the available structure     */
   BASEMAP          *basemap;       /* ptr to the basemap structure       */
   MAP_DOC          *mapdoc;        /* ptr to the map document            */
   FILE             *file;          /* file pointer                       */
   FILE_SPEC        temp_file_spec; /* file spec used to hold dted specs  */
   BOOLEAN          load_map;       /* flag to check if map was loaded    */
   BOOLEAN          problem;        /* flag to check for read problems    */
   DOUBLE           dbl_num;        /* used in byte swapping              */

   char  pHMEM      prfl_colr;      /* ptr to profile color buffer        */
   short pHMEM      prfl_hgts;      /* ptr to profile height buffer       */
   short            prior_mask;     /* product priority mask              */
   short            prod_type;      /* product type mask                  */
   short            product;        /* product exists variable            */
   short            prof;           /* profile exists variable            */
   short            shrt_num;       /* used in swapping bytes             */
   short            profile;        /* flag used to check prfl exists     */
   char             header[80], ch; /* string header placed on each file  */
   long             loop;           /* loop counter                       */
   long             lon_num;        /* used in byte swapping              */

   extern SETTINGS_TYPE settings;   /* user settings                      */
   extern SETTINGS_TYPE lastprod;   /* last product settings              */
   extern MAP_TYPE mapinfo;         /* non-muse basemap information       */

   /* get pointer to the MUSE structure */
   available_structure (&av);
   if (av->windows->current_map == (WINDOW)NULL)
   {
      xvt_dm_post_error("There is no open map window");
      return FALSE;
   }

   /* get basemap and map geometry pointers */
   mapdoc = (MAP_DOC *)xvt_vobj_get_data(av->windows->current_map);
   if (mapdoc == (MAP_DOC *)NULL)
   {
      xvt_dm_post_error("construct_product() called with NULL map_doc");
      return FALSE;
   }

   basemap = mapdoc->products->basemap;

   load_map = FALSE;
   problem  = FALSE;
   xvt_scr_set_busy_cursor();

#if XVT_OS == XVT_OS_MAC
   strcpy(prod_file_spec.name, "default.los");
   strcpy(prod_file_spec.type, "\0");
#else
   strcpy (prod_file_spec.name, "default.los");
   strcpy (prod_file_spec.type, "los");
#endif
   prod_file_spec.dir = basemap->file_spec.dir;
   xvt_fsys_set_dir(&prod_file_spec.dir);

   /* get product file name */
   status_update ("\0");
   switch (xvt_dm_post_file_open(&prod_file_spec, "Please select product file..."))
   {
      case FL_BAD:
         xvt_dm_post_error("Unable to get product file name");
         DrawMap ("Drawing basemap without Product overlay...", load_map);

         if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
         {
            if (load_dted() != STAT_SUCCESS)
               return FALSE;
         }
         return TRUE;
      case FL_CANCEL:
         DrawMap ("Drawing basemap without Product overlay...", load_map);

         if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
         {
            if (load_dted() != STAT_SUCCESS)
               return FALSE;
         }
         return TRUE;
   }

   xvt_app_process_pending_events();

   xvt_scr_set_busy_cursor();

   /* open and read product */
   file = fopen(prod_file_spec.name, "rb");
   if (file == (FILE *)NULL)
   {
      xvt_dm_post_error("Unable to open saved product file");
      fix_settings();
      DrawMap ("Drawing basemap without Product overlay...", load_map);

      if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      {
         if (load_dted() != STAT_SUCCESS)
            return FALSE;
      }
      return TRUE;
   }

   /* read the file header */
   if (fread (header, (HEADER_LEN * sizeof(char)), 1, file) != (size_t)1)
      problem = TRUE;

   header[HEADER_LEN] = (char)NULL;

   if (!problem)
      if (fread (&ch, sizeof(char), 1, file) != (size_t)1)
         problem = TRUE;

   if (!problem)
      if (fread (&ch, sizeof(char), 1, file) != (size_t)1)
         problem = TRUE;

   if (problem)
   {
      status_update("\0");
      xvt_dm_post_error("Unable to read saved product file");
      fclose (file);
      fix_settings();
      DrawMap ("Drawing basemap without Product overlay...", load_map);

      if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      {
         if (load_dted() != STAT_SUCCESS)
            return FALSE;
      }
      return TRUE;
   }

   if (strcmp (header, HEADER))
   {
      status_update("\0");
      xvt_dm_post_error("This file is not an LOS Product file");
      fclose (file);
      fix_settings();
      DrawMap ("Drawing basemap without Product overlay...", load_map);

      if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      {
         if (load_dted() != STAT_SUCCESS)
            return FALSE;
      }
      return TRUE;
   }

   /* read the product raster */
   if (raster_construct(file, &prod_raster) != STAT_SUCCESS)
   {
      status_update("\0");
      xvt_dm_post_error("Unable to read saved product raster");
      fclose (file);
      fix_settings();
      DrawMap("Drawing basemap without Product overlay...", load_map);

      if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      {
         if (load_dted() != STAT_SUCCESS)
            return FALSE;
      }
      return TRUE;
   }

   /* check product extents */
   if (!BoxWithin (prod_raster->data_hdr.extent, basemap->area->data_hdr.extent))
   {
      status_update("\0");
      xvt_dm_post_error("Saved product lies outside map geographic boundaries");
      fclose (file);
      if (raster_destruct(NULL, TRUE, &prod_raster) != STAT_SUCCESS)
         xvt_dm_post_fatal_exit("Unable to free product memory");
      prod_raster = (RASTER *)NULL;

      fix_settings();
      DrawMap ("Drawing basemap without Product overlay...", load_map);

      if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      {
         if (load_dted() != STAT_SUCCESS)
            return FALSE;
      }
      return TRUE;
   }

   if (!load_map)
      temp_file_spec = mapinfo.dted_file;

   /* read the saved user settings */
   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.start = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.end = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.observer.loc.x = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.observer.loc.y = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.observer.height = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.target.loc.x = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.target.loc.y = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.target.height = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.target.width = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        settings.range = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&lon_num, sizeof(long), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
         if (big_endian_machine())
            lon_num = swap_long (lon_num);
         settings.vert_acc = lon_num;
      }
   }

   if (!problem)
      if (fread (&prior_mask, sizeof(short), 1, file) != (size_t)1)
         problem = TRUE;

   if (!problem)
      if (fread (&prod_type, sizeof(short), 1, file) != (size_t)1)
         problem = TRUE;

   if (!problem)
      if (fread (&product, sizeof(short), 1, file) != (size_t)1)
         problem = TRUE;

   if (!problem)
      if (fread (&prof, sizeof(short), 1, file) != (size_t)1)
         problem = TRUE;

   if (!problem)
   {
      if (big_endian_machine())
      {
         prior_mask = swap_short (prior_mask);
         prod_type  = swap_short (prod_type);
         product    = swap_short (product);
         prof       = swap_short (prof);
      }
      settings.prior_mask = prior_mask;
      settings.prod_type  = prod_type;
      settings.product    = (BOOLEAN)product;
      settings.profile    = (BOOLEAN)prof;
   }

   /* read the map information */
   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        mapinfo.dist = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        mapinfo.prof_dist = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      mapinfo.t.y = settings.target.loc.y;
      mapinfo.t.x = settings.target.loc.x;
      mapinfo.o.y = settings.observer.loc.y;
      mapinfo.o.x = settings.observer.loc.x;
   }

   /* read the los or profile parameters */
   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        param.radial_spacing = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        param.grid_size_m = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        param.grid_size_dd = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        param.x_res_m = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        param.y_res_m = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&dbl_num, sizeof(DOUBLE), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
        if (big_endian_machine())
           dbl_num = swap_double (dbl_num);
        param.radial_step_size = (GFLOAT)dbl_num;
      }
   }

   if (!problem)
   {
      if (fread (&lon_num, sizeof(long), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
         if (big_endian_machine())
            lon_num = swap_long (lon_num);
         param.num_radials = lon_num;
      }
   }

   if (!problem)
   {
      if (fread (&lon_num, sizeof(long), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
         if (big_endian_machine())
            lon_num = swap_long (lon_num);
         param.nhts_in_radial = lon_num;
      }
   }

   if (!problem)
   {
      if (fread (&lon_num, sizeof(long), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
         if (big_endian_machine())
            lon_num = swap_long (lon_num);
         param.max_nhts_in_radial = lon_num;
      }
   }

   if (!problem)
   {
      if (fread (&lon_num, sizeof(long), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
         if (big_endian_machine())
            lon_num = swap_long (lon_num);
         param.radial_length = lon_num;
      }
   }

   if (!problem)
   {
      if (fread (&shrt_num, sizeof(short), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
         if (big_endian_machine())
            shrt_num = swap_short (shrt_num);
         param.radial_count = shrt_num;
      }
   }

   if (!problem)
   {
      if (fread (&profile, sizeof(short), 1, file) != (size_t)1)
         problem = TRUE;
      else
      {
         if (big_endian_machine())
            profile = swap_short (profile);
      }
   }

   if (problem)
   {
      status_update("\0");
      xvt_dm_post_error("Unable to read saved product file");
      fclose (file);
      if (raster_destruct(NULL, TRUE, &prod_raster) != STAT_SUCCESS)
         xvt_dm_post_fatal_exit("Unable to free product memory");
      prod_raster = (RASTER *)NULL;

      fix_settings();
      DrawMap ("Drawing basemap without Product overlay...", load_map);

      if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      {
         if (load_dted() != STAT_SUCCESS)
            return FALSE;
      }
      return TRUE;
   }

   /* set the appropriate DTED file specs */
   if (!load_map)
      mapinfo.dted_file = temp_file_spec;
   else
   {
      mapinfo.dted_file.dir = basemap->file_spec.dir;
      strcpy(mapinfo.dted_file.name, basemap->file_spec.name);
      strcpy(mapinfo.dted_file.type, "dte");
      mapinfo.dted_file.name[strlen(mapinfo.dted_file.name)-4] = '\0';
      strcat(mapinfo.dted_file.name, ".dte");
   }

   if (profile)
   {
      settings.profile = TRUE;

      handle.prfl = (GHANDLE) xvt_gmem_alloc ((long)(param.nhts_in_radial * sizeof(char)));
      handle.hgts = (GHANDLE) xvt_gmem_alloc ((long)(param.nhts_in_radial * sizeof(short)));

      if ((handle.prfl == (GHANDLE)NULL) || (handle.prfl == (GHANDLE)NULL))
      {
         status_update("\0");
         xvt_dm_post_error("Unable to allocate profile buffer memory");
         product_destruct(NULL, TRUE);
         fclose (file);

         fix_settings();
         DrawMap ("Drawing basemap without Product overlay...", load_map);

         if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
         {
            if (load_dted() != STAT_SUCCESS)
               return FALSE;
         }
         return TRUE;
      }

      prfl_colr = (char  pHMEM) xvt_gmem_lock (handle.prfl);
      prfl_hgts = (short pHMEM) xvt_gmem_lock (handle.hgts);

      if ((prfl_colr == (char pHMEM)NULL) || (prfl_hgts == (short pHMEM)NULL))
      {
         status_update("\0");
         xvt_dm_post_error("Unable to lock down profile buffer memory");
         product_destruct(NULL, TRUE);
         fclose(file);

         fix_settings();
         DrawMap ("Drawing basemap without Product overlay...", load_map);

         if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
         {
            if (load_dted() != STAT_SUCCESS)
               return FALSE;
         }
         return TRUE;
      }

      for (loop = 0; loop < param.nhts_in_radial; loop++)
      {
         if (!problem)
            if (fread (&(prfl_colr[loop]), sizeof(char), 1, file) != (size_t)1)
               problem = TRUE;

         if (!problem)
         {
            if (fread (&shrt_num, sizeof(short), 1, file) != (size_t)1)
               problem = TRUE;
            else
            {
               if (big_endian_machine())
                  shrt_num = swap_short (shrt_num);
               prfl_hgts[loop] = shrt_num;
            }
         }

         if (problem)
         {
            status_update("\0");
            xvt_dm_post_error("Unable to read saved product file");
            fclose (file);
            if (raster_destruct(NULL, TRUE, &prod_raster) != STAT_SUCCESS)
               xvt_dm_post_fatal_exit("Unable to free product memory");
            prod_raster = (RASTER *)NULL;

            if (xvt_gmem_unlock(handle.prfl) == FALSE)
               xvt_dm_post_fatal_exit("Unable to unlock profile color buffer");
            if (xvt_gmem_unlock(handle.hgts) == FALSE)
               xvt_dm_post_fatal_exit("Unable to unlock profile height buffer");

            if (xvt_gmem_free (handle.prfl) == FALSE)
               xvt_dm_post_fatal_exit("Unable to free profile color buffer");
            if (xvt_gmem_free (handle.hgts) == FALSE)
               xvt_dm_post_fatal_exit("Unable to free profile height buffer");

            fix_settings();
            DrawMap ("Drawing basemap without Product overlay...", load_map);

            if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
            {
               if (load_dted() != STAT_SUCCESS)
                  return FALSE;
            }
            return TRUE;
         }
      }

      if (xvt_gmem_unlock(handle.prfl) == FALSE)
         xvt_dm_post_fatal_exit("Unable to unlock profile color buffer");
      if (xvt_gmem_unlock(handle.hgts) == FALSE)
         xvt_dm_post_fatal_exit("Unable to unlock profile height buffer");
   }

   fclose (file);
   status_update("\0");

   /* check for DTED in memory */
   if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
   {
      if (load_dted() != STAT_SUCCESS)
         return FALSE;
   }

   /* display product */
   status_update("Displaying saved product...");

   display_product (prod_raster, &backgnd_raster);
   DrawMap ("\0", load_map);

   settings.profile = FALSE;

   /* make a copy of the settings */
   lastprod.start           = settings.start;
   lastprod.end             = settings.end;
   lastprod.observer.loc.x  = settings.observer.loc.x;
   lastprod.observer.loc.y  = settings.observer.loc.y;
   lastprod.observer.height = settings.observer.height;
   lastprod.target.loc.x    = settings.target.loc.x;
   lastprod.target.loc.y    = settings.target.loc.y;
   lastprod.target.height   = settings.target.height;
   lastprod.target.width    = settings.target.width;
   lastprod.range           = settings.range;
   lastprod.prior_mask      = settings.prior_mask;
   lastprod.prod_type       = settings.prod_type;

   status_update("\0");
   return TRUE;
}
#endif

int set_settings(float startaz, float endaz,
		  float obsx, float obsy, float obsheight, /* float obswidth,*/
		  float tarx, float tary, float tarheight, float tarwidth,
		  float range, long vertacc,
		  int priormask, /* unused? */
		  int prodtype, /* enum { VIS, VIP } 2 or 4 valued */
		  int prof /* viewshed or profile */)
{
  /* error checks from update_settings:
     !profile:
            xvt_dm_post_error("Azimuth Start is out of range (-360 to 360)");
            xvt_dm_post_error("Azimuth End is out of range (-359 to 360)");
         xvt_dm_post_error("Azimuth End is smaller than Azimuth Start");
         xvt_dm_post_error("Azimuth Start cannot be greater than or equal to Azimuth End for an LOS product");

     both:
            sprintf(msg, "Observer lat is out of range : %s to %s", mintext, maxtext); -90 to 90
            sprintf(msg, "Observer lon is out of range : %s to %s", mintext, maxtext); -180 to 180
         xvt_dm_post_error("Observer lies in an area where no DTED exists");
               xvt_dm_post_error("Observer lies on/beneath ground level in AMSL.  %s",
               xvt_dm_post_error("Observer height must be greater than or equal to zero");
            sprintf(msg, "Target lat is out of range : %s to %s", mintext, maxtext);
            sprintf(msg, "Target lon is out of range : %s to %s", mintext, maxtext);
         xvt_dm_post_error("Target and Observer occupy same coordinates");

     !profile:
               xvt_dm_post_error("Azimuth Start cannot be equal to Azimuth End for an LOS product (except 0 to 360)");
               xvt_dm_post_error("Target does not lie within the viewing region determined by azimuth settings");

     both:
         xvt_dm_post_error("Target lies in an area where no DTED exists");
               xvt_dm_post_error("Target lies on/beneath ground level in AMSL.  %s",
               xvt_dm_post_error("Target height must be greater than or equal to zero");
            xvt_dm_post_error("Target width must be greater than zero");
     !profile:
       get range from user setting
     both:
       compute gt_circle distance between obs and tgt
     profile:
       set range to distance
     viewshed:
       range must be >= distance
       */
   GFLOAT      xrun, yrun, xrun_m, yrun_m; /* azimuth calc variables  */
   GFLOAT      angle;                      /* more azimuth info       */
   GFLOAT      check_start, check_end;     /* used to check tgt pos   */
   GFLOAT      length;                     /* distance to target      */
  short elev;

  if (prof == 0) { /* viewshed */
    if (startaz < -360.0 || startaz > 360.0) {
      printf("Azimuth Start is out of range (-360 to 360)\n");
      return 1;
    }
    else
      settings.start = startaz;
    if (endaz < -359.0 || endaz > 360.0) {
      printf("Azimuth End is out of range (-359 to 360)\n");
      return 1;
    }
    else
      settings.end = endaz;
    if (settings.end < settings.start) {
      printf("Azimuth End is smaller than Azimuth Start\n");
      return 1;
    }
    if (settings.start >= settings.end) {
      printf("Azimuth Start cannot be greater than or equal to Azimuth End for an LOS product\n");
      return 1;
    }
  }

  if (obsx < -180.0 || obsx > 180.0) {
    printf("Observer lon is out of range\n");
    return 1;
  }
  else
    settings.observer.loc.x = obsx;
  if (obsy < -90.0 || obsy > 90.0) {
    printf("Observer lat is out of range\n");
    return 1;
  }
  else
    settings.observer.loc.y = obsy;
  elev = obsv_elev(settings.observer.loc.y, settings.observer.loc.x);
  if (elev == UNKNOWN_DTED) {
    printf("Observer lies in an area where no DTED exists\n");
    return 1;
  }
  if (obsheight < 0.0) { /* sys_prefs.hgt_rel == AMSL ??? */
    printf("Observer height must be greater than or equal to zero\n");
    return 1;
  }
  else
    settings.observer.height = obsheight;
  /* settings.observer.width = obswidth; */
  if (tarx < -180.0 || tarx > 180.0) {
    printf("Target lon is out of range\n");
    return 1;
  }
  else
    settings.target.loc.x = tarx;
  if (tary < -90.0 || tary > 90.0) {
    printf("Target lat is out of range\n");
    return 1;
  }
  else
    settings.target.loc.y = tary;
  if (settings.observer.loc.x == settings.target.loc.x &&
      settings.observer.loc.y == settings.target.loc.y) {
    printf("Target and Observer occupy same coordinates\n");
    return 1;
  }

  if (prof == 0) { /* viewshed */
      xrun  = settings.target.loc.x - settings.observer.loc.x;
      yrun  = settings.target.loc.y - settings.observer.loc.y;

      if (yrun == ZERO)
      {
         if (xrun > ZERO)
            angle = (GFLOAT) 90.0000;
         else
            angle = (GFLOAT) 270.0000;
      }
      else
      {
         xrun_m = xrun * METERS_PER_DEGREE * cos (settings.observer.loc.y / DEG_PER_RAD);
         yrun_m = yrun * METERS_PER_DEGREE;

         angle = atan2 ((GFLOAT)xrun_m, (GFLOAT)yrun_m) * 180.00 / PI;
         if (angle < ZERO)
            angle += MAX_ANGLE;
      }

      if ((settings.start == (GFLOAT)0) && (settings.end == (GFLOAT)360))
      {
	/* problem = FALSE; */
      }
      else
      {
         check_start = settings.start;

         if ((check_start == (GFLOAT)360) || (check_start == (GFLOAT)-360))
         {
            check_start = (GFLOAT)0;
            settings.start = (GFLOAT)0;
         }
  
         if (check_start < (GFLOAT)ZERO)
            check_start += (GFLOAT)360;

         check_end = settings.end;
      
         if ((check_end == (GFLOAT)360) || (check_end == (GFLOAT)-360))
         {
            check_end = (GFLOAT)0;
            settings.end = (GFLOAT)0;
         }

         if (settings.start > settings.end)
            settings.end += (GFLOAT)360;

         if (check_end < (GFLOAT)ZERO)
            check_end += (GFLOAT)360;

         if (check_start > check_end)
            check_end += (GFLOAT)360;
 
         if (1/*(!problem) && (!update)*/)
         {
            if (settings.start == settings.end)
            {
               printf("Azimuth Start cannot be equal to Azimuth End for an LOS product (except 0 to 360)\n");
	       return 1;
            }
         }

         if ((angle < check_start) || (angle > check_end))
         {
            angle += (GFLOAT)360;

            if ((angle < check_start) || (angle > check_end))
            {
               printf("Target does not lie within the viewing region determined by azimuth settings\n");
	       return 1;
	    }
	 }
      }
  }
  elev = obsv_elev(settings.target.loc.y, settings.target.loc.x);
  if (elev == UNKNOWN_DTED)
    {
      printf("Target lies in an area where no DTED exists\n");
      return 1;
    }
  if (tarheight < 0.0) {
    printf("Target height must be greater than or equal to zero\n");
    return 1;
  }
  else
    settings.target.height = tarheight;
  if (tarwidth <= 0.0) {
    printf("Target width must be greater than zero\n");
    return 1;
  }
  else
    settings.target.width = tarwidth;

  length = gt_circle(settings.target.loc, settings.observer.loc);
  if (prodtype == 0) { /* viewshed */
    if (range < length) {
      printf("Range must be at least %f meters\n", length);
      return 1;
    }
    else
      settings.range = range;
  }
  else { /* profile */
    settings.range = length;
  }

  settings.vert_acc = vertacc;
  if (priormask == 0)
    settings.prior_mask = VISIBILITY;
  else
    settings.prior_mask = HIDDEN;
  if (prodtype == 0)
    settings.prod_type = VIS;
  else
    settings.prod_type = VIP;
  settings.profile = prof;
  return 0;
}
