/*
 * Copyright (c) 1995 Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <MapUnidraw/gsview.h>
#include <MapUnidraw/mapviewstate.h>
#include <MapUnidraw/thchooser.h>
#include <InterViews/font.h>
#include <InterViews/hit.h>
#include <InterViews/layout.h>
#include <InterViews/style.h>
#include <InterViews/target.h>
#include <IV-look/choice.h>
#include <IV-look/dialogs.h>
#include <IV-look/fbrowser.h>
#include <IV-look/kit.h>

ThemeChooser::ThemeChooser(Style* s)
:InputHandler(nil, s)
{
    style_ = new Style(s);
    style_->alias("ThemeChooser");
    build();
}

ThemeChooser::~ThemeChooser() {}

GlyphIndex ThemeChooser::selected() const {
    return fbrowser_->selected();
}

static float MENU_HEIGHT = 0.5;

void ThemeChooser::build() {
    WidgetKit& wk = *WidgetKit::instance();
    LayoutKit& lk = *LayoutKit::instance();
    wk.push_style();
    wk.style(style_);
    long rows = 10;
    style_->find_attribute("rows", rows);
    const Font* f = wk.font();
    FontBoundingBox bbox;
    f->font_bbox(bbox);
    Coord height = rows * (MENU_HEIGHT*ivcm+6) + 1.0;
    if (!style_->find_attribute("width", width))
	width = 16 * f->width('m') + 3.0;
    fbrowser_ = new FileBrowser(WidgetKit::instance(), nil, nil);
    remove_all_input_handlers();
    append_input_handler(fbrowser_);

    Glyph* g = lk.vbox();
    g->append(lk.vglue(5.0, 0.0, 2.0));
    g->append(
	lk.hbox(
	    lk.vcenter(
		wk.inset_frame(
		    lk.margin(
			lk.natural_span(fbrowser_, width + 100, height), 1.0
		    )
		),
		1.0
	    ),
	    lk.hspace(4.0),
	    wk.vscroll_bar(fbrowser_->adjustable())
	)
    );
    body(
	lk.back(
	    lk.vcenter(wk.outset_frame(lk.margin(g, 5.0)), 1.0),
	    new Target(nil, TargetPrimitiveHit)
	)
    );
    focus(fbrowser_);
    wk.pop_style();
}

void ThemeChooser::append(MapTheme* theme) {
    WidgetKit& wk = *WidgetKit::instance();
    LayoutKit& lk = *LayoutKit::instance();
    ObservableGS* ogs = new ObservableGS(theme->gs());
    Glyph* entry = lk.hbox(
	lk.vcenter(lk.hnatural(wk.label(theme->name()), width)),
	lk.vcenter(new BrushView(ogs)),
	lk.hspace(4),
	lk.vcenter(new PatternView(ogs))
    );
    Glyph* label = new Target(
	lk.vmargin(lk.hmargin(entry, 3.0, 0.0, 0.0, 15.0, fil, 0.0), 2),
	TargetPrimitiveHit);
    TelltaleState* t = new TelltaleState(TelltaleState::is_enabled);
    fbrowser_->append_selectable(t);
    fbrowser_->append(new ChoiceItem(t, label, wk.bright_inset_frame(label)));
    fbrowser_->refresh();
}
