/*
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems, Cider Press
 * Copyright 1993 David B. Hollenbeck
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * VpfKit definitions
 */

#include <Map/mapcov.h>
#include <Map/mapdb.h>
#include <Map/mapelt.h>
#include <Map/mapfclass.h>
#include <Map/maplib.h>

#include <Vpf/vpfdb.h>
#include <Vpf/vpflib.h>
#include <Vpf/vpfcov.h>
#include <Vpf/vpffclass.h>

#include <VpfUnidraw/vpfkit.h>
#include <VpfUnidraw/vpfcmds.h>
#include <VpfUnidraw/vpfcomps.h>
#include <VpfUnidraw/vpfedtile.h>
#include <VpfUnidraw/vpfgaz.h>
#include <VpfUnidraw/vpftilesel.h>

#include <MapUnidraw/geoloctool.h>
#include <MapUnidraw/mapabout.h>
#include <MapUnidraw/mapcamera.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/mapexport.h>
#include <MapUnidraw/mapinspect.h>
#include <MapUnidraw/mappoint.h>
#include <MapUnidraw/mapproj.h>
#include <MapUnidraw/mapstatecmds.h>
#include <MapUnidraw/mapstates.h>

#include <OverlayUnidraw/ovcmds.h>
#include <OverlayUnidraw/ovcomps.h>
#include <OverlayUnidraw/ovdoer.h>
#include <OverlayUnidraw/oved.h>
#include <OverlayUnidraw/ovimport.h>
#include <OverlayUnidraw/ovprint.h>
#include <OverlayUnidraw/ovviewer.h>

#define iv2_6_compatible 1
#include <UniIdraw/idcmds.h>
#include <UniIdraw/idkybd.h>
#undef iv2_6_compatible

#include <Unidraw/unidraw.h>
#include <Unidraw/editor.h>
#include <Unidraw/ctrlinfo.h>
#include <Unidraw/uctrls.h>
#include <Unidraw/keymap.h>
#include <Unidraw/kybd.h>
#include <Unidraw/catalog.h>
#include <Unidraw/viewer.h>

#include <Unidraw/Graphic/pspaint.h>

#include <Unidraw/Tools/magnify.h>

#include <Unidraw/Commands/catcmds.h>

#include <IVGlyph/gdialogs.h>

#include <IV-look/kit.h>

#include <InterViews/background.h>
#include <InterViews/dialog.h>
#include <InterViews/layout.h>
#include <InterViews/label.h>
#include <InterViews/action.h>
#include <InterViews/patch.h>
#include <InterViews/display.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/window.h>

#include <IV-X11/xdisplay.h>
#include <IV-X11/xcolor.h>
#include <IV-X11/xpattern.h>

#include <OS/math.h>

#include <IVGlyph/exportchooser.h>
#include <IVGlyph/figure.h>
#include <IVGlyph/toolbutton.h>

#include <stdlib.h>
#include <string.h>

implementTable(VpfFClassMenuTable,VpfFeatureClass*,MenuItem*)
implementTable(VpfFClassTable,VpfFeatureClass*,boolean)

/******************************************************************************/

declareActionCallback(VpfKit)
implementActionCallback(VpfKit)

VpfKit* VpfKit::_vpfkit = nil;

VpfKit::VpfKit()
: MapKit()
{
    WidgetKit& kit = *WidgetKit::instance();
    Style* s = kit.style();

    _vpfgaz = new VpfGazetteer(this, s);
    _vpfgaz->ref();
    _fcmenutable = new VpfFClassMenuTable(100);
    _fctable = new VpfFClassTable(100);
}

VpfKit::~VpfKit() {
    _vpfgaz->unref();
    delete _fcmenutable;
    delete _fctable;
}

VpfKit* VpfKit::Instance() {
    if (!_vpfkit)
	_vpfkit = new VpfKit();
    return _vpfkit;
}

Glyph* VpfKit::MakeToolbar() {
    WidgetKit& kit = *WidgetKit::instance();
    const LayoutKit& layout = *LayoutKit::instance();
    Style* s = kit.style();

    ToolButton* magnify;
    ToolButton* mapselect;
    ToolButton* inspect;
#ifdef HAVE_ACE
    ToolButton* camerapos;
    ToolButton* camerapath;
#endif
    ToolButton* geoloc;
    _toolbars = layout.deck(2);

    PolyGlyph* vb = layout.vbox(6);

    TelltaleGroup* tg = new TelltaleGroup();

    Glyph* mag = kit.label("Magnify");
    Glyph* mps = kit.label("Select");
    Glyph* ins = kit.label("Inspect");
    Glyph* til = kit.label("TileSel");
#ifdef HAVE_ACE
    Glyph* campos = kit.label("CameraPos");
    Glyph* campth = kit.label("CameraPath");
#endif
    Glyph* geolocl = kit.label("GeoLoc");

    Coord maxwidth = 0;
    Requisition req;
    maxwidth = Math::max((mag->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((mps->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((ins->request(req), req.x_requirement().natural()),
			 maxwidth);
#ifdef HAVE_ACE
    maxwidth = Math::max((campos->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((campth->request(req), req.x_requirement().natural()),
			 maxwidth);
#endif
    maxwidth = Math::max((geolocl->request(req), req.x_requirement().natural()),
			 maxwidth);
    maxwidth = Math::max((til->request(req), req.x_requirement().natural()),
			 maxwidth);

    vb->append(magnify = MakeTool(new MagnifyTool(new ControlInfo("Magnify", KLBL_MAGNIFY,CODE_MAGNIFY)),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(mag)), tg));
    vb->append(mapselect = MakeTool(new MapSelectTool(new ControlInfo("Select", "s", "s")),
				  layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						 layout.hcenter(mps)), tg));
    vb->append(inspect = MakeTool(new MapInspectTool(((MapEditor*)_ed)->selstate(),
					   new ControlInfo("Inspect", "i", "i")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(ins)), tg));
#ifdef HAVE_ACE
    vb->append(camerapos = MakeTool(new CameraPosTool(new ControlInfo("CameraPos", "c", "c")),
				 layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(campos)), tg));
    vb->append(camerapath = MakeTool(new CameraPathTool(new ControlInfo("CameraPath", "C", "C")),
				 layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(campth)), tg));
#endif
    vb->append(geoloc = MakeTool(new GeoLocTool(new ControlInfo("GeoLoc", "G", "G")),
				 layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
						layout.hcenter(geolocl)), tg));
    vb->append(MakeTool(new TileSelectTool(new ControlInfo("TileSel")),
			layout.overlay(layout.hcenter(layout.hspace(maxwidth)),
				       layout.hcenter(til)), tg));
    _toolbars->append(vb);

    vb = layout.vbox(4);
    vb->append(magnify);
    vb->append(mapselect);
    vb->append(inspect);
#ifdef HAVE_ACE
    vb->append(camerapos);
    vb->append(camerapath);
#endif
    vb->append(geoloc);
    _toolbars->append(vb);
    _toolbars->flip_to(1);
    _toolbar = new Patch(_toolbars);

    return layout.hbox(
	layout.vflexible(
	    layout.vnatural(
		new Background(
		    layout.vcenter(
			_toolbar
		    ),
		    unidraw->GetCatalog()->FindColor("#aaaaaa")
		),
		550
	    )
	)
    );
}

void VpfKit::toolbar0() {
    _toolbars->flip_to(0);
    _toolbar->redraw();
}

void VpfKit::toolbar1() {
    _toolbars->flip_to(1);
    _toolbar->redraw();
}

Glyph* VpfKit::MakeMenus()
{
    LayoutKit& lk = *LayoutKit::instance();
    Menu* menubar_ = WidgetKit::instance()->menubar();
    
    menubar_->append_item(MakeFileMenu());
    menubar_->append_item(MakeViewMenu());
    menubar_->append_item(MakeMapMenu());
    Resource::ref(menubar_);
    return LayoutKit::instance()->vbox(lk.hnatural(menubar_, 600));
}

MenuItem * VpfKit::MakeFileMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    Style * style = new Style(Session::instance()->style());
    
    MenuItem *mbi = kit.menubar_item(kit.label("File"));
    mbi->menu(kit.pulldown());
    MakeMenu(mbi, new MapAboutCmd(new ControlInfo("About vpfviewer", KLBL_ABOUT, CODE_ABOUT)),
	     "About ivmaps vpfviewer");
    MakeMenu(mbi, new OvNewCompCmd(new ControlInfo("New", KLBL_NEWCOMP, CODE_NEWCOMP),
				 new VpfIdrawComp),
	     "New   ");
    MakeMenu(mbi, new OvRevertCmd(new ControlInfo("Revert", KLBL_REVERT, CODE_REVERT)),
	     "Revert   ");
    MakeMenu(mbi, new VpfOpenCmd(new ControlInfo("Open...", KLBL_VIEWCOMP, CODE_VIEWCOMP)),
	     "Open...   ");
    MakeMenu(mbi, new OvSaveCompCmd(new ControlInfo("Save", KLBL_SAVECOMP, CODE_SAVECOMP)),
	     "Save   ");
    MakeMenu(mbi, new OvSaveCompAsCmd(new ControlInfo("Save As...",
					    KLBL_SAVECOMPAS,
					    CODE_SAVECOMPAS)),
	     "Save As...   ");
    MakeMenu(mbi, new OvPrintCmd(new ControlInfo("Print...", KLBL_PRINT, CODE_PRINT)),
	     "Print...   ");
    style->attribute("subcaption", "Export selected map features to file:");
    style->attribute("open", "Export");
    const char *formats[] = {"idraw", "drawtool"};
    ExportChooser* echooser = new ExportChooser(".", WidgetKit::instance(), style,
				 formats, sizeof(formats)/sizeof(char*), nil, nil, true, false);
    Resource::ref(echooser);
    MakeMenu(mbi, new MapExportCmd(new ControlInfo("Export Graphic...",
						  "^X", "\030"), 
				   echooser),
	     "Export Graphic...   ");
    MakeMenu(mbi, new OvWindowDumpAsCmd(new ControlInfo("Dump Window As..."
						  )),
	     "Dump Window As...   ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new OvQuitCmd(new ControlInfo("Quit", KLBL_QUIT, CODE_QUIT)),
	     "Quit   ");
    
    return mbi;
}

MenuItem * VpfKit::MakeViewMenu() {
  return MapKit::MakeViewMenu();
}

void VpfKit::MakeCheckMenu(MenuItem* mbi, Command* scmd, Command* hcmd,
			   const char* label, VpfFeatureClass* fc) {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* item;
    boolean oldval;
    _ed->GetKeyMap()->Register(new CommandControl(scmd->GetControlInfo()));
    scmd->SetEditor(_ed);
    _ed->GetKeyMap()->Register(new CommandControl(hcmd->GetControlInfo()));
    hcmd->SetEditor(_ed);
    MenuItem* menu_item = kit.check_menu_item(kit.label(label));
    _fcmenutable->insert(fc, menu_item);
    ShowHideDoer* doer = new ShowHideDoer(scmd, hcmd, menu_item->state());
    menu_item->action(new ActionCallback(ShowHideDoer)(doer, &ShowHideDoer::Do));
    if (_fctable->find(oldval, fc) && oldval)
	menu_item->state()->set(TelltaleState::is_chosen, true);
    mbi->menu()->append_item(menu_item);
}

MenuItem* VpfKit::MakeLayerMenus() {
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* showi = kit.menu_item(kit.label(   "Layers                     "));
    Menu* show = kit.pullright();
    showi->menu(show);
    VpfEditor* ed = (VpfEditor*)GetEditor();
    VpfDatabase* db = ed->GetMapDatabase();
    VpfLibrary* lib = db->library(ed->libstate()->maplib());
    if (lib) {
	int maxcovlen =0;
	for (int ic = 0; ic < lib->ncovs(); ic++) {
	    VpfCoverage* cov = lib->coverage(ic);
	    int len = strlen(cov->description());
	    if (len > maxcovlen)
		maxcovlen = len;
	}
	for (int ic = 0; ic < lib->ncovs(); ic++) {
	    VpfCoverage* cov = lib->coverage(ic);
	    char covdesc[maxcovlen+1];
	    memset(covdesc, ' ', maxcovlen);
	    covdesc[maxcovlen] = '\0';
	    strncpy(covdesc, cov->description(), strlen(cov->description()));
	    MenuItem* showcovi = kit.menu_item(kit.label(covdesc));
	    Menu* showcovm = kit.pullright();
	    showcovi->menu(showcovm);
	    int maxfclen =0;
	    for (int ifc = 0; ifc < cov->nfcls(); ifc++) {
		VpfFeatureClass* fc = cov->feature_class(ifc);
		int len = strlen(fc->description());
		if (len > maxfclen)
		    maxfclen = len;
	    }
	    for (int ifc = 0; ifc < cov->nfcls(); ifc++) {
		VpfFeatureClass* fc = cov->feature_class(ifc);
		MapShowCmd* showcmd = new MapShowCmd(new ControlInfo(fc->description()), false);
		showcmd->SetLibName(lib->name());
		showcmd->SetCovName(cov->name());
		showcmd->SetFClassName(fc->name());
		MapHideCmd* hidecmd = new MapHideCmd(new ControlInfo(fc->description()), false);
		hidecmd->SetLibName(lib->name());
		hidecmd->SetCovName(cov->name());
		hidecmd->SetFClassName(fc->name());
		char fcdesc[maxfclen+1];
		memset(fcdesc, ' ', maxfclen);
		fcdesc[maxfclen] = '\0';
		strncpy(fcdesc, fc->description(), strlen(fc->description()));
		// if fc shown set check for menu
		MakeCheckMenu(showcovi, showcmd, hidecmd, fcdesc, fc);
	    }
	    show->append_item(showcovi);
	}
    }
    return showi;
}

MenuItem* VpfKit::MakeShowLayerMenus() {
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* showi = kit.menu_item(kit.label("Show Layer                 "));
    Menu* show = kit.pullright();
    showi->menu(show);
    VpfEditor* ed = (VpfEditor*)GetEditor();
    VpfDatabase* db = ed->GetMapDatabase();
    if (db) {
    VpfLibrary* lib = db->library(ed->libstate()->maplib());
    if (lib) {
	int maxcovlen =0;
	for (int ic = 0; ic < lib->ncovs(); ic++) {
	    VpfCoverage* cov = lib->coverage(ic);
	    int len = strlen(cov->description());
	    if (len > maxcovlen)
		maxcovlen = len;
	}
	maxcovlen += 5;
	for (int ic = 0; ic < lib->ncovs(); ic++) {
	    VpfCoverage* cov = lib->coverage(ic);
	    if (strcmp(cov->name(), "libref")==0 ||
		strcmp(cov->name(), "tileref")==0)
		continue;
	    char covdesc[maxcovlen+1];
	    memset(covdesc, ' ', maxcovlen);
	    covdesc[maxcovlen] = '\0';
	    strncpy(covdesc, "Show ", 5);
	    strncpy(covdesc+5, cov->description(), strlen(cov->description()));
	    MenuItem* showcovi = kit.menu_item(kit.label(covdesc));
	    Menu* showcovm = kit.pullright();
	    showcovi->menu(showcovm);
	    int maxfclen =0;
	    for (int ifc = 0; ifc < cov->nfcls(); ifc++) {
		VpfFeatureClass* fc = cov->feature_class(ifc);
		int len = strlen(fc->description());
		if (len > maxfclen)
		    maxfclen = len;
	    }
	    maxfclen += 5;
	    for (int ifc = 0; ifc < cov->nfcls(); ifc++) {
		VpfFeatureClass* fc = cov->feature_class(ifc);
		MapShowCmd* showcmd = new MapShowCmd(new ControlInfo(fc->description()), false);
		showcmd->SetLibName(lib->name());
		showcmd->SetCovName(cov->name());
		showcmd->SetFClassName(fc->name());
		char fcdesc[maxfclen+1];
		memset(fcdesc, ' ', maxfclen);
		fcdesc[maxfclen] = '\0';
		strncpy(fcdesc, "Show ", 5);
		strncpy(fcdesc+5, fc->description(), strlen(fc->description()));
		MakeMenu(showcovi, showcmd, fcdesc);
	    }
	    show->append_item(showcovi);
	}
    }
    }
    return showi;
}

MenuItem* VpfKit::MakeHideLayerMenus() {
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* hidei = kit.menu_item(kit.label("Hide Layer                 "));
    Menu* hide = kit.pullright();
    hidei->menu(hide);
    VpfEditor* ed = (VpfEditor*)GetEditor();
    VpfDatabase* db = ed->GetMapDatabase();
    if (db) {
    VpfLibrary* lib = db->library(ed->libstate()->maplib());
    if (lib) {
	int maxcovlen =0;
	for (int ic = 0; ic < lib->ncovs(); ic++) {
	    VpfCoverage* cov = lib->coverage(ic);
	    int len = strlen(cov->description());
	    if (len > maxcovlen)
		maxcovlen = len;
	}
	maxcovlen += 5;
	for (int ic = 0; ic < lib->ncovs(); ic++) {
	    VpfCoverage* cov = lib->coverage(ic);
	    if (strcmp(cov->name(), "libref")==0 ||
		strcmp(cov->name(), "tileref")==0)
		continue;
	    char covdesc[maxcovlen+1];
	    memset(covdesc, ' ', maxcovlen);
	    covdesc[maxcovlen] = '\0';
	    strncpy(covdesc, "Hide ", 5);
	    strncpy(covdesc+5, cov->description(), strlen(cov->description()));
	    MenuItem* hidecovi = kit.menu_item(kit.label(covdesc));
	    Menu* hidecovm = kit.pullright();
	    hidecovi->menu(hidecovm);
	    int maxfclen =0;
	    for (int ifc = 0; ifc < cov->nfcls(); ifc++) {
		VpfFeatureClass* fc = cov->feature_class(ifc);
		int len = strlen(fc->description());
		if (len > maxfclen)
		    maxfclen = len;
	    }
	    maxfclen += 5;
	    for (int ifc = 0; ifc < cov->nfcls(); ifc++) {
		VpfFeatureClass* fc = cov->feature_class(ifc);
		MapHideCmd* hidecmd = new MapHideCmd(new ControlInfo(fc->description()), false);
		hidecmd->SetLibName(lib->name());
		hidecmd->SetCovName(cov->name());
		hidecmd->SetFClassName(fc->name());
		char fcdesc[maxfclen+1];
		memset(fcdesc, ' ', maxfclen);
		fcdesc[maxfclen] = '\0';
		strncpy(fcdesc, "Hide ", 5);
		strncpy(fcdesc+5, fc->description(), strlen(fc->description()));
		MakeMenu(hidecovi, hidecmd, fcdesc);
	    }
	    hide->append_item(hidecovi);
	}
    }
    }
    return hidei;
}

void VpfKit::FClassState(VpfFeatureClass* fc, boolean val) {
    MenuItem* item;
    boolean oldval;
    if (_fcmenutable->find(item, fc))
	item->state()->set(TelltaleState::is_chosen, val);
    _fctable->insert(fc, val);
}

MenuItem* VpfKit::MakeLibraryMenu() {
    WidgetKit& kit = *WidgetKit::instance();
    MenuItem* libi = kit.menu_item(kit.label(  "Library                    "));
    Menu* libm = kit.pullright();
    libi->menu(libm);
    VpfEditor* ed = (VpfEditor*)GetEditor();
    VpfDatabase* db = ed->GetMapDatabase();
    if (db) {
    int maxliblen = 0;
    for (int il = 0; il < db->nlibs(); il++) {
	VpfLibrary* lib = db->library(il);
	int len = strlen(lib->name());
	if (len > maxliblen)
	    maxliblen = len;
    }
    TelltaleGroup* tg = new TelltaleGroup();
    for (int il = 0; il < db->nlibs(); il++) {
	VpfLibrary* lib = db->library(il);
	char libname[maxliblen+1];
	memset(libname, ' ', maxliblen);
	libname[maxliblen] = '\0';
	strncpy(libname, lib->name(), strlen(lib->name()));
	MenuItem* menu_item = kit.radio_menu_item(tg, libname);
	if (strcmp(lib->name(), ed->libstate()->maplib())==0)
	    menu_item->state()->set(TelltaleState::is_chosen, true);
	MapLibCmd* cmd = new MapLibCmd(new ControlInfo(""), lib->name());
	cmd->SetEditor(ed);
	MacroCmd* mcmd = new MacroCmd(ed, cmd,
				      new VpfToolbarCmd(ed, lib->tiled()),
				      new VpfUpdateMenusCmd(ed));
	CommandPusher* pusher = new CommandPusher(mcmd);
	menu_item->action(new ActionCallback(CommandPusher)
			  (pusher, &CommandPusher::Push));
	libm->append_item(menu_item);
    }
    }
    return libi;
}

MenuItem* VpfKit::MakeMapMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();

    MenuItem *mbi = kit.menubar_item(kit.label("Map"));
    mbi->menu(kit.pulldown());
    // 0
    MakeMenu(mbi, new MapProjectionCmd(new ControlInfo("Projection...")),
	     "Projection...              ");

    // 1
    mbi->menu()->append_item(kit.menu_item_separator());

    // 2
    _libmenu = MakeLibraryMenu();
    mbi->menu()->append_item(_libmenu);

    // 3
    _showlayermenu = MakeShowLayerMenus();
    mbi->menu()->append_item(_showlayermenu);

    // 4
    _hidelayermenu = MakeHideLayerMenus();
    mbi->menu()->append_item(_hidelayermenu);

    MenuItem* menu_item = kit.menu_item(kit.label("Gazeteer...                "));
    menu_item->action(new ActionCallback(VpfKit)(this, &VpfKit::gaz));
    mbi->menu()->append_item(menu_item);

    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new VpfGraticuleCmd(new ControlInfo("Graticule")),
	     "Graticule                  ");

    _mapmenu = mbi;
    return mbi;
}

void VpfKit::UpdateLibraryMenu() {
    _mapmenu->menu()->replace_item(2, MakeLibraryMenu());
}

void VpfKit::UpdateLayerMenu() {
    _mapmenu->menu()->replace_item(3, MakeShowLayerMenus());
    _mapmenu->menu()->replace_item(4, MakeHideLayerMenus());
}

void VpfKit::gaz() {
  VpfEditor* editor = (VpfEditor*)GetEditor();
  VpfDatabase* db = editor->GetMapDatabase();
  MapLibState* state = editor->libstate();
  VpfLibrary* lib = db->library(state->maplib());
  if (lib && lib->gazette())
    _vpfgaz->post_for(_ed->GetWindow());
  else {
    char buf[BUFSIZ];
    if (lib && lib->name()) 
      sprintf(buf, "%s library doesn't have gazetteer", lib->name());
    else
      sprintf(buf, "no library defined");
    GAcknowledgeDialog::post(_ed->GetWindow(), buf);
  }
}

MapLibrary* VpfShowTilesCmd::choosetiledlib(MapDatabase* mapdb) {
    int cnt = 0;
    for (int i = 0; i < mapdb->nlibs(); i++) {
	if (mapdb->library(i)->tiled())
	    cnt++;
    }
    if (cnt == 0) {
	// no tiled libraries found
	return nil;
    }
    else if (cnt == 1) {
	// just one, use it.
	for (int i = 0; i < mapdb->nlibs(); i++)
	    if (mapdb->library(i)->tiled())
		return mapdb->library(i);
    }
    else {
	// more than one, pick one
	StringList* list = new StringList;
	String* stri;
	for (int i = 0; i < mapdb->nlibs(); i++) {
	    if (mapdb->library(i)->tiled()) {
		stri = new String(mapdb->library(i)->name());
		list->append(*stri);
	    }
	}
	StrChooser* chooser = new StrChooser(list, new String("Select Tiled Library"),
					 WidgetKit::instance(),
					 Session::instance()->style());
	MapEditor* editor = (MapEditor*)GetEditor();
	while (chooser->post_for((Window*)editor->GetWindow())) {
	    String str;
	    if (chooser->selected() > -1)
		str = list->item_ref(chooser->selected());
	    else
		continue;
	    const char* name = str.string();
	    delete chooser;
	    delete list;
	    return mapdb->library(name);
	    break;
	}
	delete chooser;
	delete list;
	return nil;
    }
    return nil;
}

VpfTileKit* VpfTileKit::_vpftilekit = nil;

VpfTileKit::VpfTileKit() {
}

VpfTileKit* VpfTileKit::Instance() {
    if (!_vpftilekit)
	_vpftilekit = new VpfTileKit();
    return _vpftilekit;
}

MenuItem* VpfTileKit::MakeMapMenu() {
    WidgetKit& kit = *WidgetKit::instance();

    MenuItem *mbi = kit.menubar_item("Map");
    mbi->menu(kit.pulldown());
    MakeMenu(mbi, new MapProjectionCmd(new ControlInfo("Projection...")),
	     "Projection...              ");
    MakeMenu(mbi, new PointCmd(new ControlInfo("Place Geographic Point...  ")),
	     "Place Geographic Point...  ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new MapLibCmd(new ControlInfo("Select VPF Library...      ")),
	     "Select VPF Library...      ");
    MakeMenu(mbi, new MapCovCmd(new ControlInfo("Select VPF Coverage...     ")),
	     "Select VPF Coverage...     ");
    MakeMenu(mbi, new MapFClassCmd(new ControlInfo("Select VPF Feature Class...")),
	     "Select VPF Feature Class...");
    MakeMenu(mbi, new MapShowCmd(new ControlInfo("Show Map Feature Class     ")), 
	     "Show Map Feature Class     ");
    MakeMenu(mbi, new MapHideCmd(new ControlInfo("Hide Map Feature Class     ")),
	     "Hide Map Feature Class     ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MenuItem* menu_item = kit.menu_item(kit.label("Gazeteer"));
    menu_item->action(new ActionCallback(VpfKit)(this, &VpfKit::gaz));
    mbi->menu()->append_item(menu_item);

    return mbi;
}

Glyph* VpfTileKit::MakeStates() {
    Catalog* catalog = unidraw->GetCatalog();
    ((VpfTileEditor*)_ed)->_fclassstate = new MapFClassState(catalog->GetAttribute("fclass"),
				 ((VpfTileEditor*)_ed)->_mapdb->library(catalog->GetAttribute("library"))->
				 coverage(catalog->GetAttribute("coverage"))->
				 feature_class(catalog->GetAttribute("fclass"))->
				 description());
    ((VpfTileEditor*)_ed)->_covstate = new MapCovState(((VpfTileEditor*)_ed)->_fclassstate,
				catalog->GetAttribute("coverage"),
				((VpfTileEditor*)_ed)->_mapdb->library(((VpfTileEditor*)_ed)->_parent->libstate()->maplib())->
				coverage(catalog->GetAttribute("coverage"))->
				description());
    ((VpfTileEditor*)_ed)->_libstate = new MapLibState(((VpfTileEditor*)_ed)->_covstate, 
						       ((VpfTileEditor*)_ed)->_parent->libstate()->maplib());
    ((VpfTileEditor*)_ed)->_dbstate = new MapDbState(((VpfTileEditor*)_ed)->_libstate, catalog->GetAttribute("database"));
    ((VpfTileEditor*)_ed)->_projstate = new ProjectionState();
    ((VpfTileEditor*)_ed)->_selstate = new MapSelectionState("");

    NameView* fclassview = new NameView(((VpfTileEditor*)_ed)->_fclassstate);
    NameView* covview = new NameView(((VpfTileEditor*)_ed)->_covstate);
    NameView* libview = new NameView(((VpfTileEditor*)_ed)->_libstate);
    NameView* dbview = new NameView(((VpfTileEditor*)_ed)->_dbstate);
    NameView* projview = new NameView(((VpfTileEditor*)_ed)->_projstate);
    NameView* selview = new NameView(((VpfTileEditor*)_ed)->_selstate);

    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    return kit.outset_frame(
	lk.hmargin(
	    lk.vbox(
		lk.vmargin(lk.hbox(dbview, projview), 2),
		lk.vmargin(libview, 2),
		lk.vmargin(selview, 2)
	    ),
	    4,0,0,0.0,fil,0.0
	)
    );
}

void VpfKit::tiledismiss() {
    _tiledialog->dismiss(true);
}

void VpfKit::ExplainTileSel(char* libname) {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& wk = *WidgetKit::instance();
    char lab[256];
    sprintf(lab, "You must sweep an area of interest with the TileSelect tool within the %s library first.",
	    libname);
    Action* dis = new ActionCallback(VpfKit)(
	this, &VpfKit::tiledismiss
    );
    Button* but = wk.push_button("OK", dis);
    Glyph* g = wk.outset_frame(lk.margin(lk.vbox(lk.hcenter(wk.label(lab)), lk.vspace(10), lk.hcenter(but)), 10));
    _tiledialog = new Dialog(g, wk.style());
    _tiledialog->post_for(GetEditor()->GetWindow());
}

void VpfKit::InitMembers (OverlayComp* comp) {
  OverlayKit::InitMembers(comp);
}
